#include <qfile.h>
#include <qtextstream.h>
#include <kdebug.h>

#include "xmlinfo.h"

XMLInfo::XMLInfo()
{
    QDomElement root = defs.createElement( "xmel" );
    defs.appendChild( root );

    QDomElement doctype = defs.createElement( "doctype" );
    root.appendChild( doctype );

    info = defs.createElement( "info" );
    doctype.appendChild( info );

    tags = defs.createElement( "tags" );
    doctype.appendChild( tags );

    entities = defs.createElement( "entities" );
    doctype.appendChild( entities );
}

XMLInfo::~XMLInfo()
{
}

bool XMLInfo::load( const QString &filename )
{
    QFile f( filename );

    if ( !f.open( IO_ReadOnly ) )
	return false;

    if ( !doc.setContent( &f ) ) {
      f.close();
      return false;
    }

    f.close();
    return true;
}

bool XMLInfo::saveInfo( const QString &filename )
{
    QFile f( filename );

    if ( !f.open( IO_WriteOnly ) )
	return false;

    createOutput();

    QTextStream ts( &f );
    ts << defs.toString();
    f.close();

    return true;
}

void XMLInfo::createOutput()
{
    QMap<QString,QDomElement>::Iterator it;
    for ( it = knownTags.begin() ; it != knownTags.end() ; ++it ) {
	tags.appendChild( it.data() );
    }
}

//
// Methods that walk the DOM tree 
//

void XMLInfo::walkTree()
{
    processDocument( doc );
    QDomElement root = doc.documentElement();
    processElement( root );
    walkElement( root );
}

void XMLInfo::walkElement( const QDomElement &elem )
{
    QDomNode n = elem.firstChild();
    while( !n.isNull() ) {
	QDomElement e = n.toElement();
	if( !e.isNull() ) {
	    processElement( e );
	    walkElement( e );
	}
	else {
	    QDomEntity ent = n.toEntity();   
	    if( !ent.isNull() )
		processEntity( ent );
	}
	n = n.nextSibling();
    }
}

//
// Methods that construct the description data
//

void XMLInfo::processDocument( QDomDocument &doc )
{
    QString s;
    QDomDocumentType dt = doc.doctype();

    QDomElement name = defs.createElement( "name" );
    s = dt.name();
    name.appendChild( doc.createTextNode( s ) );
    info.appendChild( name );

    QDomElement pub = defs.createElement( "publicid" );
    s = dt.publicId();
    pub.appendChild( doc.createTextNode( s ) );
    info.appendChild( pub );

    QDomElement sys = defs.createElement( "systemid" );
    s = dt.systemId();
    sys.appendChild( doc.createTextNode( s ) );
    info.appendChild( sys );
}

void XMLInfo::processElement( const QDomElement &elem )
{
    QString tag( elem.tagName() );

    QDomElement def;
    if ( knownTags.contains( tag ) ) {
//	def = knownTags[ tag ];
	return;
    }
    else {
	def = defs.createElement( "tag" );
	def.setAttribute( "name", tag );
    }

    //
    // Record attributes
    //
    QDomNamedNodeMap attrs = elem.attributes();
    for ( uint i = 0 ; i < attrs.count() ; i++ ) {
	QDomAttr attr = attrs.item(i).toAttr();
	if ( !attr.isNull() ) {
	    QDomElement a = defs.createElement( "attr" );
	    a.setAttribute( "name", attr.name() );
	    def.appendChild( a );
	}
    }

#if 0
    //
    // Record child elements
    //
    QDomNode n = elem.firstChild();
    while( !n.isNull() ) {
	QDomElement e = n.toElement();
	if( !e.isNull() ) {
	    QDomElement c = defs.createElement( "child" );
	    c.setAttribute( "name", e.tagName() );
	    def.appendChild( c );
	}
	n = n.nextSibling();
    }
#endif

    knownTags[ tag ] = def;
}

void XMLInfo::processEntity( const QDomEntity &ent )
{
    QDomElement e = defs.createElement( "entity" );
    e.setAttribute( "name", ent.notationName() );
    entities.appendChild( e );
}

// Local Variables:
// c-basic-offset: 4
// End:
